import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Properties;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

public class Main {
	private static Properties localProperties;
	private static Properties localVersions;
	private static Properties remoteProperties;
	private static Properties remoteVersions;
	private final static String REMOTE_PROPS_NAME = "remote.update.properties";
	private final static String LOCAL_PROPS_NAME = "local.update.properties";
	private final static String LOCAL_VERSIONS_NAME = "local.versions";
	 final static String REMOTE_VERSIONS_NAME = "remote.versions";
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				} catch (Exception ex) {
				}
				final PropertiesLoader loader = PropertiesLoader.getInstance();
				
				try {
					localProperties = loader.loadProperties(LOCAL_PROPS_NAME);
					try {
						localVersions = loader.loadProperties(LOCAL_VERSIONS_NAME);
					} catch (IOException ex) {
						localVersions = new Properties();
					}
					
					String updateURLtemp = localProperties.getProperty("updateurl", null);
					if (updateURLtemp == null) {
						JOptionPane.showMessageDialog(null, "Error reading local 'updateurl' property, make sure it's in " + LOCAL_PROPS_NAME, "Failed to read local properties", JOptionPane.ERROR_MESSAGE);
						System.exit(0);
					}
					if (!updateURLtemp.endsWith("/")) updateURLtemp += "/";
					final String updateURL = updateURLtemp;
					
					try {
						remoteProperties = loader.loadProperties(new URL(updateURL + REMOTE_PROPS_NAME));
						
						try {
							remoteVersions = loader.loadProperties(new URL(updateURL + REMOTE_VERSIONS_NAME));
		
							final String ip = remoteProperties.getProperty("ip", null);
							if (ip == null) {
								JOptionPane.showMessageDialog(null, "Error reading remote 'ip' property, make sure it's in " + updateURL + REMOTE_PROPS_NAME, "Failed to read remote properties", JOptionPane.ERROR_MESSAGE);
								System.exit(0);
							}
							final String port = remoteProperties.getProperty("port", null);
							if (port == null) {
								JOptionPane.showMessageDialog(null, "Error reading remote 'port' property, make sure it's in " + updateURL + REMOTE_PROPS_NAME, "Failed to read remote properties", JOptionPane.ERROR_MESSAGE);
								System.exit(0);
							}
							final String name = remoteProperties.getProperty("name", null);
							if (name == null) {
								JOptionPane.showMessageDialog(null, "Error reading remote 'name' property, make sure it's in " + updateURL + REMOTE_PROPS_NAME, "Failed to read remote properties", JOptionPane.ERROR_MESSAGE);
								System.exit(0);
							}
							final UpdaterFrame updaterFrame = new UpdaterFrame(name + " Updater");
						
						// If there's an update
							new Thread(new Runnable() {
								@Override
								public void run() {
									ArrayList<String> updatedFiles = new ArrayList<String>();
									for (Object remoteFileObject : remoteVersions.keySet()) {
										String remoteFile = (String)remoteFileObject;
										
										if (localVersions.get(remoteFile) == null || !localVersions.get(remoteFile).equals(remoteVersions.get(remoteFile))) {
											updatedFiles.add(remoteFile);
										}
									}
									
									if (updatedFiles.size() > 0) {
										updaterFrame.setVisible(true);
									
									
										for (String updatedFile : updatedFiles) {
											if (updaterFrame.downloadUpdates(updateURL, updatedFile) == true) {
												localVersions.put(updatedFile, remoteVersions.get(updatedFile));
											}
										}
										
										loader.saveProperties(localVersions, LOCAL_VERSIONS_NAME);
										updaterFrame.setVisible(false);
									}
									
									final String[] cmd = { "ody/ody.exe", ip, port, name.replace(" ", "") };
									try {
										Runtime.getRuntime().exec(cmd);
										System.exit(0);
									} catch (Exception ex) {
										JOptionPane.showMessageDialog(null, ex.getMessage());
									}
								}
							}).start();
						// Failed to load remote versions
						} catch (IOException ex) {
							JOptionPane.showMessageDialog(null, ex.getMessage(), "Failed to load remote versions", JOptionPane.ERROR_MESSAGE);
							System.exit(0);
						}
				
					// Failed to load remote properties
					} catch (IOException ex) {
						JOptionPane.showMessageDialog(null, ex.getMessage(), "Failed to load remote properties", JOptionPane.ERROR_MESSAGE);
						System.exit(0);
					}
				// Failed to load local properties
				} catch (IOException ex) {
					JOptionPane.showMessageDialog(null, ex.getMessage(), "Failed to load local properties", JOptionPane.ERROR_MESSAGE);
					System.exit(0);
				}
			}
		});
	}
}
