
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URL;
import java.util.Properties;

public class PropertiesLoader {
	private static final PropertiesLoader instance = new PropertiesLoader();
	
	private PropertiesLoader() {
	}
	
	public static PropertiesLoader getInstance() {
		return instance;
	}
	
	public Properties loadProperties(String propertiesFile) throws IOException {
		 Properties properties = new Properties();
		 FileInputStream file = new FileInputStream(propertiesFile);
		 properties.load(file);
		 file.close();
		 return properties;
	 }
	
	public Properties loadProperties(URL propertiesURL) throws IOException {
		InputStream is = propertiesURL.openStream(); // throws an IOException
		BufferedReader br = new BufferedReader(new InputStreamReader(is));
		String line;
		Properties props = new Properties();

		while ((line = br.readLine()) != null) {
			if (line.contains("=")) {
				String parts[] = line.split("=");
				props.put(parts[0].trim(), parts[1].trim());
			}
		}
		
		is.close();
		br.close();
		return props;
	}
	
	public void saveProperties(Properties p, String fileName) {
		OutputStream outPropFile;
		try {
			File propsFile = new File(fileName);
			if(!propsFile.exists()) {
				propsFile.createNewFile();
			} 
			outPropFile = new FileOutputStream(fileName);
			p.store(outPropFile, "Updated version number");
			outPropFile.close();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}

	}

}
