
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.zip.ZipException;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

public class UpdaterFrame extends JFrame {
	private static final long serialVersionUID = 1L;
	private JProgressBar progressBar = new JProgressBar();
	private JLabel topStatusText = new JLabel("Downloading...");
	private JLabel statusText = new JLabel("0.0 MB/ 0.0 MB");
	private final String DIST_FOLDER = "ody";

	public UpdaterFrame(String title) {
		this.setTitle(title);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setupGUI();
		this.pack();
		this.setLocationRelativeTo(null);
	}

	private void setupGUI() {
		this.setPreferredSize(new Dimension(400, 200));
		GridBagConstraints c = new GridBagConstraints();
		progressBar.setPreferredSize(new Dimension(250, 20));
		progressBar.setStringPainted(true);
		this.setLayout(new GridBagLayout());
		c.insets = new Insets(5, 5, 5, 5);
		c.gridy = 0;
		this.add(progressBar, c);
		c.gridy = 1;
		c.insets = new Insets(5, 5, 1, 5);
		this.add(topStatusText, c);
		c.insets = new Insets(1, 5, 5, 5);
		c.gridy = 2;
		this.add(statusText, c);
	}
	
	private void setMaxProgress(final int progress) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				progressBar.setMaximum(progress);
			}
		});
	}
	
	private void setProgress(final int progress) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				progressBar.setValue(progress);
			}
		});
	}
	
	private void setStatus(final String status) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				statusText.setText(status);
			}
		});
	}
	
	private void setTopStatus(final String status) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				topStatusText.setText(status);
			}
		});
	}

	public boolean downloadUpdates(String updateURL, String updateFile) {
		boolean success = false;
		//progressBar.setIndeterminate(false);
		// If it doesn't exist, download it
		new File(DIST_FOLDER).mkdir();
		setTopStatus("Downloading " + updateFile + "...");
		try {
			URL website = new URL(updateURL + updateFile);
			BufferedInputStream in = null;
			FileOutputStream fout = null;
			try {
				final int downloadSize = website.openConnection().getContentLength();
				setMaxProgress(downloadSize);
				in = new BufferedInputStream(website.openStream());
				fout = new FileOutputStream(DIST_FOLDER + "/" + updateFile);
				byte data[] = new byte[1024];
				int count;
				int bytesRead = 0;
				while ((count = in.read(data, 0, 1024)) != -1) {
					float mbRead = (float)Math.round(((float)bytesRead/(float)1048576) * 10) / 10;
					float mbTotal = (float)Math.round(((float)downloadSize/(float)1048576) * 10) / 10;
					setProgress(bytesRead);
					setStatus(mbRead  + " MB / " + mbTotal + " MB");
					fout.write(data, 0, count);
					bytesRead += 1024;
					if (bytesRead > downloadSize) bytesRead = downloadSize;
				}
			} finally {
				if (in != null)
					in.close();
				if (fout != null)
					fout.close();
			}
			if (updateFile.endsWith(".zip")) {
				setTopStatus("Extracting " + updateFile + "...");
				ZipUtil.getInstance().extractFolder(DIST_FOLDER + "/" + updateFile, DIST_FOLDER);
			}
			success = true;
		} catch (ZipException ex) {
			JOptionPane.showMessageDialog(null, ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			ex.printStackTrace();
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(null, ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			ex.printStackTrace();
		}
		
		return success;

	}

}
